from collections import defaultdict
from classes.Flow import *

class Paragraph:
  def __init__(self, paragraphID, sentences, sequencemarkers, only_constraints):
    self.paragraphID = paragraphID
    self.sentences = sentences
    self.sequencemarkers = sequencemarkers
    self.only_constraints = only_constraints
    self.obligations = self.calculate_obligations(only_constraints)    
    self.flows = self.calculate_flows()   # flow = sequential order relation
    
  def __repr__(self):
    result = "id: %s\n" % (self.paragraphID) + "\n"
    for s in self.sentence:
      result += repr(s) + "\n"
    return s

  # take all clauses in lemmatized bag-of-words representation
  def calculate_obligations(self, only_constraints):
    result = list()
    for sentence in self.sentences:
      if(self.only_constraints):
        if(sentence.constraint is False):
          continue
        for clause in sentence.clauses:
          result.append(clause) 
        if(not result):
          result.append(self.sentence)
      else:
        for clause in sentence.clauses:
          result.append(clause) 
        if(not result):
          result.append(self.sentence)
    return result

  ## CALCULATE FLOWS ############### START
  def calculate_flows(self):
    if(len(self.obligations) == 1):
      return None
    result = list()
    for idx, obligation in enumerate(self.obligations):
      if(self.exists_flow_intra(obligation)):
        tmp = self.extract_intra_flow(obligation)
        result.append(Flow(obligation._id, tmp[0], tmp[1], obligation.stopwords))
      elif(self.exists_flow(obligation, self.obligations[idx])):
        result.append(Flow(obligation._id, obligation, self.obligations[idx], obligation.stopwords))
    return result

  def exists_flow_intra(self, obligation):
    for x in obligation.clause:
      if(x.text in self.sequencemarkers and x.i == 0 and x.text == "After"):
        return True
    return False
        
  def extract_intra_flow(self, obligation):
    condition = list()
    consequence = list()
    indices = list()
    for t in obligation.clause[0].subtree:
      condition.append(t)
      indices.append(t.i)
    for t in obligation.clause:
      if(t.i not in indices):
        consequence.append(t)
    return (condition, consequence)

  def exists_flow(self, obl1, obl2):
    for x in obl1.clause:
      if(x.text in self.sequencemarkers and x.i !=0):
        return True
    return False
  ## CALCULATE FLOWS ############### END    
